#include "DEM.h"
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>

#define DEBUG_Grid2
#define PRINT_Elevations

void generate_vertices(int Datafile, DEM_record **Record, int LL_x, int LL_y,
                       int UR_x, int UR_y);
void generate_triangle(int Datafile, int M, int N);

main(int argc, char *argv[])
{
   DEM         *Area;
   DEM_record **Records;

   int          LL_x;
   int          LL_y;

   int          UR_x;
   int          UR_y;

   int          Datafile;
   char         Data_filename[80];

   int          i;

   double       Y_Factor;
   double       Y_Base;
   double       Y_Base_Plus_Y_Length;
   double       Max_x;
   double       Max_y;
   double       Min_x;
   double       Min_y;

   if (argc < 6)
      printf("usage: %s <datafile> LLx LLy UR_x UR_y\n", argv[0]);

   Area = read_DEM_file(argv[1]);
   Records = Area -> records;

   sscanf(argv[2], "%d", &LL_x);
   sscanf(argv[3], "%d", &LL_y);

   sscanf(argv[4], "%d", &UR_x);
   sscanf(argv[5], "%d", &UR_y);

   if ((LL_x < 0) || (LL_y < 0) || (UR_x < 0) || (UR_y < 0)
       || (LL_x > UR_x) || (LL_y > UR_y) || (UR_x > Area -> Header -> N - 1)
       || (UR_y > Records[0] -> M - 1))
   {
      printf(stderr, "Illegal coordinate!");
      exit(1);
   }

#if 0
   print_DEM(Area);
   print_DEM_header(Area -> Header);
#endif

   Max_x = Area -> Header -> Ground_Coordinates[0][0];
   Min_x = Area -> Header -> Ground_Coordinates[0][0];
   Max_y = Area -> Header -> Ground_Coordinates[0][1];
   Min_y = Area -> Header -> Ground_Coordinates[0][1];

   for (i = 1; i < 4; i++)
      if (Max_x < Area -> Header -> Ground_Coordinates[i][0])
         Max_x = Area -> Header -> Ground_Coordinates[i][0];
      else if (Min_x > Area -> Header -> Ground_Coordinates[i][0])
         Min_x = Area -> Header -> Ground_Coordinates[i][0];
      else if (Max_y < Area -> Header -> Ground_Coordinates[i][1])
         Max_y = Area -> Header -> Ground_Coordinates[i][1];
      else if (Min_y > Area -> Header -> Ground_Coordinates[i][1])
         Min_y = Area -> Header -> Ground_Coordinates[i][1];

   Y_Factor = (Max_y - Min_y) / (Area -> Header -> M - 1);

   fprintf(stderr, "(0, 0) is at (%g, %g)\n",
           Records[0] -> Coordinate[0] / 3600,
           Records[0] -> Coordinate[1] / 3600);

   /* assume square patches */

   fprintf(stderr, "(%d, %d) is at (%g, %g)\n",
           Area -> Header -> N - 1,
           Area -> Header -> M - 1,
           Records[Area -> Header -> N - 1] -> Coordinate[0] / 3600,
           ((Records[Area -> Header -> N - 1] -> Coordinate[1] + Max_y - Min_y)
            / 3600));

   Y_Base = Y_Factor * LL_y;
   Y_Base_Plus_Y_Length = Y_Factor * UR_y;

   fprintf(stderr, "(LL_x = %d, LL_y = %d) is at (%g, %g)\n", LL_x, LL_y,
           Records[LL_x] -> Coordinate[0] / 3600,
           (Records[LL_x] -> Coordinate[1] + Y_Base) / 3600);

   fprintf(stderr, "(UR_x = %d, UR_y = %d) is at (%g, %g)\n", UR_x, UR_y,
           Records[UR_x] -> Coordinate[0] / 3600,
           (Records[UR_x] -> Coordinate[1] + Y_Base_Plus_Y_Length) / 3600);

   sprintf(Data_filename, "%s.dat\0", argv[1]);

   if ((Datafile = open(Data_filename, O_CREAT | O_WRONLY | O_TRUNC,
                        S_IREAD | S_IWRITE)) == -1)
   {
      perror(argv[0]);
      exit(1);
   }

   generate_vertices(Datafile, Records, LL_x, LL_y, UR_x, UR_y);

   generate_triangle(Datafile, UR_y - LL_y + 1, UR_x - LL_x + 1);

   close(Datafile);
}

